/*
 * PM - 2017
 * lab0.c
 */

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <stdlib.h>

#define OFF     0
#define RED     1
#define GREEN   2
#define YELLOW  3
#define BLINK_TIME  10

int dice;
unsigned int mat[7][7];

int old_value = RED;

int line_RED = 2;
int col_RED = 0;
int selected = 0;
int selected_line_RED;
int selected_col_RED;
int initial_line_RED = 2;
int initial_col_RED = 0;

int nr_RED_players = 0;

int initial_LineG = 4;
int initial_ColG = 6;

/* 
SHIFT REGISTER 1 - coloane matrice (anoduri) 
	PA0 - clk
	PA1 - latch
	PA2 - data

SHIFT REGISTER 2 - linii matrice (catoduri)
	PA3 - clk
	PA4 - latch
	PA5 - data

*/

int digitArray[7] = {0b00000000, 0b10000100, 0b11010011, 0b11010110, 0b10110100, 0b01110110, 0b01110111};

/* Simuleaza aruncarea cu zarul */
void roll_the_dice(void)
{
	dice = rand() % 6 + 1;
	PORTC = digitArray[dice];
}

/* Initializare matrice initiala */
void initial_matrix(void) 
{
	int i, j;
	for (i = 0; i < 7; i++)
		for (j = 0; j < 7; j++)
			mat[i][j] = 0;

	/* RED Player B */
	mat[0][0] = RED;
	mat[0][1] = RED;
	mat[1][0] = RED;

	/* RED Player A */
	mat[2][0] = RED;

	/* GREEN Player B*/
	mat[5][6] = GREEN;
	mat[6][5] = GREEN;
	mat[6][6] = GREEN;

	/* GREEN Playes A */
	mat[4][6] = GREEN;
}

/* Initializare porturi intrare/iesire */
void init(void)
{
	/* Setare pini de intrare butoane deplasare */
	DDRB &= ~((1 << PB0) | (1 << PB1) | (1 << PB2) | (1 << PB3) | (1 << PB4) | (1 << PB5) | (1 << PB6) | (1 << PB7));

	/* Setare pini de intrare pentru select */
	DDRA &= ~((1 << PA6) | (1 << PA7));

	/* Setare pin de intrare pentru zar */
	DDRD &= ~(1 << PD1);
	
	/* Activare rezistente de pull-up */
	PORTB |= (1 << PB0) | (1 << PB1) | (1 << PB2) | (1 << PB3) | (1 << PB4) | (1 << PB5) | (1 << PB6) | (1 << PB7);
	PORTA |= (1 << PA6) | (1 << PA7);
	PORTD |= (1 << PD1);

	/* Setare pini de iesire pentru cele 2 registre de deplasare */
	DDRA |= (1 << PA0) | (1 << PA1) | (1 << PA2) | (1 << PA3) | (1 << PA4) | (1 << PA5);

	/* Setare pini de iesire pentru display */
	DDRC |= (1 << PC0) | (1 << PC1) | (1 << PC2) | (1 << PC3) | (1 << PC4) | (1 << PC5) | (1 << PC6) | (1 << PC7);

	/* Shift register 1 */
	PORTA &= ~(1 << PA0); //CLK
	PORTA &= ~(1 << PA1); //LATCH
	PORTA &= ~(1 << PA2); //DATA

	/* Shift register 2 */
	PORTA &= ~(1 << PA3); //CLK
	PORTA &= ~(1 << PA4); //LATCH
	PORTA &= ~(1 << PA5); //DATA

	/* Display */
	PORTC &= ~(1 << PC0);
	PORTC &= ~(1 << PC1);
	PORTC &= ~(1 << PC2);
	PORTC &= ~(1 << PC3);
	PORTC &= ~(1 << PC4);
	PORTC &= ~(1 << PC5);
	PORTC &= ~(1 << PC6);
	PORTC &= ~(1 << PC7);
}

/* Incarcare bit shift register 1 */
void load_bit_to_register_1(unsigned int bit)
{
	/* Set bit */
	if (bit == 1)
		PORTA |= (1 << PA2);
	else
		PORTA &= ~(1 << PA2);
			
	_delay_us(10);

	/* Load bit - clk front crescator */
	PORTA |= (1 << PA0);

	_delay_us(10);

	/* Reset clk */
	PORTA &= ~(1 << PA0);
}

/* Incarcare bit shift register 2 */
void load_bit_to_register_2(unsigned int bit)
{
	/* Set bit */
	if (bit == 1)
		PORTA |= (1 << PA5);
	else
		PORTA &= ~(1 << PA5);
		
	_delay_us(10);
	
	/* Load bit - clk front crescator */
	PORTA |= (1 << PA3);
	
	_delay_us(10);
	
	/* Reset clk */
	PORTA &= ~(1 << PA3);
}

void load_color_of_led(unsigned int R, unsigned int G)
{
	load_bit_to_register_1(G);
	load_bit_to_register_1(R);
}

/* Incarcare coloana curenta */
void load_LEDS_column_colors_for_given_row(int row)
{
	int i = 6;

	/* Dezactivare latch */
	PORTA &= ~(1 << PA1);

	/* Incarcare coloana in registru de deplasare */
	while (i > -1) {
		
		int color = mat[row][i];

		if (color == RED) {
			load_color_of_led(1, 0);
		}
		else if (color == GREEN) {
			load_color_of_led(0, 1);
		}
		else if (color == YELLOW) {
			load_color_of_led(1, 1);
		}
		else {
			load_color_of_led(0, 0);
		}

		i--;
			
	}

	/* Activare latch */
	PORTA |= (1 << PA1);
} 

/* Afisare matrice */
void print_game_matrix(void)
{
	int i;
	/* Incarcare linii si coloane
	 * corespunzatoare fiecarei linii
	 */
	for (i = 0; i < 7; i++) {
	
		PORTA &= ~(1 << PA4);
			
		int j = 6;
		while (j > -1) {
			if (j == i) {
				load_bit_to_register_2(0);
			}
			else {
				load_bit_to_register_2(1);
			}
	
			j--;
		}
			
		load_LEDS_column_colors_for_given_row(i);
		
		PORTA |= (1 << PA4);
	}
}

/* Returneaza butonul apasat in functie de pinul de input activ
	
	e - zar
	o - eroare

	RED Player
		w - up
		d - right
		s - down
		a - left
		q - select

	GREEN Player
		i - up
		l - right
		k - down
		j - left
		u - select  
 */
char get_button_pressed(void)
{
	if ((PINB & (1 << PB0)) == 0) {
		_delay_ms(200);
		return 'w';
	}
	else
		if ((PINB & (1 << PB1)) == 0) {
			_delay_ms(200);
			return 'd';
		}
		else
			if ((PINB & (1 << PB2)) == 0) {
				_delay_ms(200);
				return 's';
			}
			else
				if ((PINB & (1 << PB3)) == 0) {
					_delay_ms(200);
					return 'a';
				}
				else
					if ((PINA & (1 << PA7)) == 0) {
						_delay_ms(200);
						return 'q';
					}
					else
						if ((PIND & (1 << PD1)) == 0) {
							_delay_ms(200);
							return 'e';
						}
						else
							return 'o';
					
}

void move_up_RED(void)
{
	col_RED++;
}

void move_right_RED(void)
{
	line_RED++;
}

void move_down_RED(void)
{
	col_RED--;
}

void move_left_RED(void)
{
	line_RED--;
}

/* Dupa ce jucatorul a selectat pinul pe care il doreste mutat -> mutare automata nr zar */
void move_pin_RED(void)
{
	int moved = dice;
	old_value = OFF;

	while (moved != 0) {
		int old_line_RED = line_RED;
		int old_col_RED = col_RED;

		switch (line_RED)
		{
			case 0:
				if (col_RED == 4)
					move_right_RED();
				else
					move_up_RED();
				break;
			case 1:
				if (col_RED == 2)
					move_left_RED();
				else
					move_right_RED();
				break;
			case 2:
				if (col_RED == 2)
					move_left_RED();
				else
					if (col_RED == 6)
						move_right_RED();
					else
						move_up_RED();
				break;
			case 3:
				if (col_RED == 6)
					move_right_RED();
				break;
			case 4:
				if (col_RED == 0)
					move_left_RED();
				else
					if (col_RED == 4)
						move_right_RED();
					else
						move_down_RED();
				break;
			case 5:
				if (col_RED == 4)
					move_right_RED();
				else
					move_left_RED();
				break;
			case 6:
				if (col_RED == 2)
					move_left_RED();
				else
					move_down_RED();
				break;
		}

		mat[old_line_RED][old_col_RED] = old_value;
		old_value = mat[line_RED][col_RED];
		mat[line_RED][col_RED] = RED;

		print_game_matrix();
		moved--;
	}
}

int get_final_postition_RED(void)
{
	int moved = dice;
	while (moved != 0) {
		switch (line_RED)
		{
			case 0:
				if (col_RED == 4)
					selected_line_RED++;
				else
					selected_col_RED++;
				break;
			case 1:
				if (col_RED == 2)
					selected_line_RED--;
				else
					selected_line_RED++;
				break;
			case 2:
				if (col_RED == 2)
					selected_line_RED--;
				else
					if (col_RED == 6)
						selected_line_RED++;
					else
						selected_col_RED++;
				break;
			case 3:
				if (col_RED == 6)
					selected_line_RED++;
				break;
			case 4:
				if (col_RED == 0)
					selected_line_RED--;
				else
					if (col_RED == 4)
						selected_line_RED++;
					else
						selected_col_RED--;
				break;
			case 5:
				if (col_RED == 4)
					selected_line_RED++;
				else
					selected_line_RED--;
				break;
			case 6:
				if (col_RED == 2)
					selected_line_RED--;
				else
					selected_col_RED--;
				break;
		}

		moved--;
		if (selected_line_RED == 3 && selected_col_RED == 0 && moved != 0)
			return moved;
	}

	return moved;
}

/* Mutare pion RED dupa selectie */
void make_move_RED(void)
{
	//on column with filling pins 1 and 2
	if (selected_line_RED == 6 && selected_col_RED == 3) {
		//am dat 1
		if (dice == 1) {
			//daca pinul e liber -> muta piesa pe pin
			if (mat[4][3] == OFF) {
				mat[line_RED][col_RED] = OFF;
				line_RED = 4;
				col_RED = 3;
				mat[line_RED][col_RED] = RED;
			} // altfel, verifica cate piese are dincolo de aceasta granita -> piese care trebuie sa ajunga pe pinii 3 si 4
			else {
				// am mai putin de 2 player dincolo -> pot sa mut piesa
				if (nr_RED_players < 2) {
					nr_RED_players++;
					move_pin_RED();
				}
			}
		}

		else if (dice == 2) {
			//daca pinul e liber -> muta piesa pe pin
			if (mat[5][3] == OFF) {
				mat[line_RED][col_RED] = OFF;
				line_RED = 5;
				col_RED = 3;
				mat[line_RED][col_RED] = RED;
			} // altfel, verifica cate piese are dincolo de aceasta granita -> piese care trebuie sa ajunga pe pinii 3 si 4
			else {
				// am mai putin de 2 player dincolo -> pot sa mut piesa
				if (nr_RED_players < 2) {
					nr_RED_players++;
					move_pin_RED();
				}
			}
		}

		//nu am nimerit niciunul din pinii care ar trebui completati aici -> verifica daca pot sa merg mai departe
		else {
			if (nr_RED_players < 2) {
				nr_RED_players++;
				move_pin_RED();
			}
		}
	}

	else if (selected_line_RED == 3 && selected_col_RED == 0) {
		if (dice == 3) {
			if (mat[3][2] == OFF) {
				mat[line_RED][col_RED] = OFF;
				line_RED = 3;
				col_RED = 2;
				mat[line_RED][col_RED] = RED;
			}
		}

		else if (dice == 4) {
			if (mat[3][1] == OFF) {
				mat[line_RED][col_RED] = OFF;
				line_RED = 3;
				col_RED = 1;
				mat[line_RED][col_RED] = RED;
			}
		}
	}

	//sunt in dreapta primei coloane de pini -> trebuie sa verific daca pot sa trec
	else if (selected_col_RED > 3) {
		get_final_postition_RED();

		//am trecut de coloana -> verific numarul de playeri
		if (selected_col_RED < 3) {
			if (nr_RED_players < 2) {
				nr_RED_players++;
				move_pin_RED();
			}
		}
		else {
			move_pin_RED();
		}
	}

	//sunt in stanga
	else if (selected_col_RED < 3) {
		int moved = get_final_postition_RED();
		//moved > 0, zarul prea mare si depaseste locul unde trebuie sa ajung eu
		if (moved == 0)
			move_pin_RED();
	}

	selected = 0;
	line_RED = initial_line_RED;
	col_RED = initial_col_RED;
} 

/* Rulare joc */
void ludo() {
	char c;

	while(1) {
		int max;
		c = get_button_pressed();

		int old_line_RED = line_RED;
		int old_col_RED = col_RED;

		switch (c)
		{
			case 'e':
				roll_the_dice();
				break;
			case 'w':
				max = 4;
				if (line_RED == 2 || line_RED == 3 || line_RED == 4)
					max = 6;

				if (col_RED < max) {
					if (col_RED == 2) {
						if (line_RED == 0 || line_RED == 6)
							move_up_RED();
					}
					else {
						move_up_RED();
					}
				}
				break;
			case 'd':
				max = 4;
				if (col_RED == 2 || col_RED == 3 || col_RED == 4)
					max = 6;
				
				if (line_RED < max) {
					if (col_RED == 2 || col_RED == 4)
						if (line_RED <= 1 || line_RED >= 4)
							move_right_RED();

					if (col_RED == 0 && line_RED == 3)
						move_right_RED();

					if (col_RED == 6)
						if (line_RED == 2 || line_RED == 3)
							move_right_RED();
				}
				break;
			case 's':
				if (col_RED > 0)
					move_down_RED();
				break;
			case 'a':
				if (line_RED > 0)
					move_left_RED();
				break;
			case 'q':
				if (old_value == RED) {
					selected = 1;
					selected_line_RED = line_RED;
					selected_col_RED = col_RED;
					make_move_RED();
				}
				break;
			case 'o':
				break;
		}

		if (selected == 0)
			if (old_line_RED != line_RED || old_col_RED != col_RED) {
				mat[old_line_RED][old_col_RED] = old_value;
				old_value = mat[line_RED][col_RED];

				mat[line_RED][col_RED] = RED;
			}

		print_game_matrix();
	}
}

int main()
{
	/* Initializare stare initiala a jocului */
	initial_matrix();

	/* Initializare pini */
	init();

	/* Rulare joc */
	ludo();

	return 0;
}
